@@ -1,6 +1,6 @@
 /* connection.h - definition for the psycopg connection type
  *
- * Copyright (C) 2003-2010 Federico Di Gregorio <fog@debian.org>
+ * Copyright (C) 2003-2019 Federico Di Gregorio <fog@debian.org>
  *
  * This file is part of psycopg.
  *
@@ -38,6 +38,12 @@ extern "C" {
 #define ISOLATION_LEVEL_READ_COMMITTED      1
 #define ISOLATION_LEVEL_REPEATABLE_READ     2
 #define ISOLATION_LEVEL_SERIALIZABLE        3
+#define ISOLATION_LEVEL_DEFAULT             5
+
+/* 3-state values on/off/default */
+#define STATE_OFF               0
+#define STATE_ON                1
+#define STATE_DEFAULT           2
 
 /* connection status */
 #define CONN_STATUS_SETUP       0
@@ -71,24 +77,24 @@ extern HIDDEN PyTypeObject connectionType;
 
 struct connectionObject_notice {
     struct connectionObject_notice *next;
-    const char *message;
+    char *message;
 };
 
-typedef struct {
+/* the typedef is forward-declared in psycopg.h */
+struct connectionObject {
     PyObject_HEAD
 
     pthread_mutex_t lock;   /* the global connection lock */
 
     char *dsn;              /* data source name */
-    char *critical;         /* critical error on this connection */
+    char *error;            /* temporarily stored error before raising */
     char *encoding;         /* current backend encoding */
-    char *codec;            /* python codec name for encoding */
 
     long int closed;          /* 1 means connection has been closed;
                                  2 that something horrible happened */
     long int mark;            /* number of commits/rollbacks done so far */
     int status;               /* status of the connection */
-    XidObject *tpc_xid;       /* Transaction ID in two-phase commit */
+    xidObject *tpc_xid;       /* Transaction ID in two-phase commit */
 
     long int async;           /* 1 means the connection is async */
     int protocol;             /* protocol version */
@@ -102,11 +108,12 @@ typedef struct {
      * for a green connection. If NULL, the connection is idle. */
     PyObject *async_cursor;
     int async_status;         /* asynchronous execution status */
+    PGresult *pgres;          /* temporary result across async calls */
 
     /* notice processing */
     PyObject *notice_list;
-    PyObject *notice_filter;
     struct connectionObject_notice *notice_pending;
+    struct connectionObject_notice *last_notice;
 
     /* notifies */
     PyObject *notifies;
@@ -120,7 +127,24 @@ typedef struct {
 
     int autocommit;
 
-} connectionObject;
+    PyObject *cursor_factory;    /* default cursor factory from cursor() */
+
+    /* Optional pointer to a decoding C function, e.g. PyUnicode_DecodeUTF8 */
+    PyObject *(*cdecoder)(const char *, Py_ssize_t, const char *);
+
+    /* Pointers to python encoding/decoding functions, e.g.
+     * codecs.getdecoder('utf8') */
+    PyObject *pyencoder;        /* python codec encoding function */
+    PyObject *pydecoder;        /* python codec decoding function */
+
+    /* Values for the transactions characteristics */
+    int isolevel;
+    int readonly;
+    int deferrable;
+
+    /* the pid this connection was created into */
+    pid_t procpid;
+};
 
 /* map isolation level values into a numeric const */
 typedef struct {
@@ -130,30 +154,31 @@ typedef struct {
 
 /* C-callable functions in connection_int.c and connection_ext.c */
 HIDDEN PyObject *conn_text_from_chars(connectionObject *pgconn, const char *str);
+HIDDEN PyObject *conn_encode(connectionObject *self, PyObject *b);
+HIDDEN PyObject *conn_decode(connectionObject *self, const char *str, Py_ssize_t len);
 HIDDEN int  conn_get_standard_conforming_strings(PGconn *pgconn);
-HIDDEN int  conn_get_isolation_level(connectionObject *self);
+HIDDEN PyObject *conn_pgenc_to_pyenc(const char *encoding, char **clean_encoding);
 HIDDEN int  conn_get_protocol_version(PGconn *pgconn);
 HIDDEN int  conn_get_server_version(PGconn *pgconn);
-HIDDEN PGcancel *conn_get_cancel(PGconn *pgconn);
 HIDDEN void conn_notice_process(connectionObject *self);
 HIDDEN void conn_notice_clean(connectionObject *self);
 HIDDEN void conn_notifies_process(connectionObject *self);
-HIDDEN int  conn_setup(connectionObject *self, PGconn *pgconn);
+RAISES_NEG HIDDEN int conn_setup(connectionObject *self);
 HIDDEN int  conn_connect(connectionObject *self, long int async);
 HIDDEN void conn_close(connectionObject *self);
-HIDDEN int  conn_commit(connectionObject *self);
-HIDDEN int  conn_rollback(connectionObject *self);
-HIDDEN int  conn_set_session(connectionObject *self, const char *isolevel,
-                                 const char *readonly, const char *deferrable,
-                                 int autocommit);
-HIDDEN int  conn_set_autocommit(connectionObject *self, int value);
-HIDDEN int  conn_switch_isolation_level(connectionObject *self, int level);
-HIDDEN int  conn_set_client_encoding(connectionObject *self, const char *enc);
+HIDDEN void conn_close_locked(connectionObject *self);
+RAISES_NEG HIDDEN int  conn_commit(connectionObject *self);
+RAISES_NEG HIDDEN int  conn_rollback(connectionObject *self);
+RAISES_NEG HIDDEN int conn_set_session(connectionObject *self, int autocommit,
+        int isolevel, int readonly, int deferrable);
+RAISES_NEG HIDDEN int  conn_set_client_encoding(connectionObject *self, const char *enc);
 HIDDEN int  conn_poll(connectionObject *self);
-HIDDEN int  conn_tpc_begin(connectionObject *self, XidObject *xid);
-HIDDEN int  conn_tpc_command(connectionObject *self,
-                             const char *cmd, XidObject *xid);
+RAISES_NEG HIDDEN int  conn_tpc_begin(connectionObject *self, xidObject *xid);
+RAISES_NEG HIDDEN int  conn_tpc_command(connectionObject *self,
+                             const char *cmd, xidObject *xid);
 HIDDEN PyObject *conn_tpc_recover(connectionObject *self);
+HIDDEN void conn_set_result(connectionObject *self, PGresult *pgres);
+HIDDEN void conn_set_error(connectionObject *self, const char *msg);
 
 /* exception-raising macros */
 #define EXC_IF_CONN_CLOSED(self) if ((self)->closed > 0) { \
