@@ -2,7 +2,7 @@
 #
 # types_extras.py - tests for extras types conversions
 #
-# Copyright (C) 2008-2010 Federico Di Gregorio  <fog@debian.org>
+# Copyright (C) 2008-2019 Federico Di Gregorio  <fog@debian.org>
 #
 # psycopg2 is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published
@@ -13,22 +13,30 @@
 # ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 # FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 # License for more details.
-from __future__ import with_statement
 
 import re
-import sys
+import json
+import uuid
 import warnings
 from decimal import Decimal
 from datetime import date, datetime
 from functools import wraps
 from pickle import dumps, loads
 
-from testutils import (unittest, skip_if_no_uuid, skip_before_postgres,
-    ConnectingTestCase, decorate_all_tests, py3_raises_typeerror, slow)
+import unittest
+from .testutils import (PY2, text_type, skip_if_no_uuid, skip_before_postgres,
+    ConnectingTestCase, py3_raises_typeerror, slow, skip_from_python)
 
 import psycopg2
 import psycopg2.extras
 import psycopg2.extensions as ext
+from psycopg2._json import _get_json_oids
+from psycopg2.extras import (
+    CompositeCaster, DateRange, DateTimeRange, DateTimeTZRange, HstoreAdapter,
+    Inet, Json, NumericRange, Range, RealDictConnection,
+    register_composite, register_hstore, register_range,
+)
+from psycopg2.tz import FixedOffsetTimezone
 
 
 class TypesExtrasTests(ConnectingTestCase):
@@ -41,7 +49,6 @@ class TypesExtrasTests(ConnectingTestCase):
 
     @skip_if_no_uuid
     def testUUID(self):
-        import uuid
         psycopg2.extras.register_uuid()
         u = uuid.UUID('9c6d5a77-7256-457e-9461-347b4358e350')
         s = self.execute("SELECT %s AS foo", (u,))
@@ -52,7 +59,6 @@ class TypesExtrasTests(ConnectingTestCase):
 
     @skip_if_no_uuid
     def testUUIDARRAY(self):
-        import uuid
         psycopg2.extras.register_uuid()
         u = [uuid.UUID('9c6d5a77-7256-457e-9461-347b4358e350'),
              uuid.UUID('9c6d5a77-7256-457e-9461-347b4358e352')]
@@ -94,7 +100,6 @@ class TypesExtrasTests(ConnectingTestCase):
         self.failUnless(s is None)
 
     def test_inet_conform(self):
-        from psycopg2.extras import Inet
         i = Inet("192.168.1.0/24")
         a = psycopg2.extensions.adapt(i)
         a.prepare(self.conn)
@@ -122,10 +127,10 @@ class TypesExtrasTests(ConnectingTestCase):
         s = self.execute("""SELECT '{"(1,2)","(3,4)"}' AS foo""")
         self.failUnless(s == """{"(1,2)","(3,4)"}""")
 
+
 def skip_if_no_hstore(f):
     @wraps(f)
     def skip_if_no_hstore_(self):
-        from psycopg2.extras import HstoreAdapter
         oids = HstoreAdapter.get_oids(self.conn)
         if oids is None or not oids[0]:
             return self.skipTest("hstore not available in test database")
@@ -136,11 +141,9 @@ def skip_if_no_hstore(f):
 
 class HstoreTestCase(ConnectingTestCase):
     def test_adapt_8(self):
-        if self.conn.server_version >= 90000:
+        if self.conn.info.server_version >= 90000:
             return self.skipTest("skipping dict adaptation with PG pre-9 syntax")
 
-        from psycopg2.extras import HstoreAdapter
-
         o = {'a': '1', 'b': "'", 'c': None}
         if self.conn.encoding == 'UTF8':
             o['d'] = u'\xe0'
@@ -162,11 +165,9 @@ class HstoreTestCase(ConnectingTestCase):
             self.assertQuotedEqual(ii[3], b"('d' => '" + encc + b"')")
 
     def test_adapt_9(self):
-        if self.conn.server_version < 90000:
+        if self.conn.info.server_version < 90000:
             return self.skipTest("skipping dict adaptation with PG 9 syntax")
 
-        from psycopg2.extras import HstoreAdapter
-
         o = {'a': '1', 'b': "'", 'c': None}
         if self.conn.encoding == 'UTF8':
             o['d'] = u'\xe0'
@@ -178,9 +179,9 @@ class HstoreTestCase(ConnectingTestCase):
         m = re.match(br'hstore\(ARRAY\[([^\]]+)\], ARRAY\[([^\]]+)\]\)', q)
         self.assert_(m, repr(q))
 
-        kk = m.group(1).split(b", ")
-        vv = m.group(2).split(b", ")
-        ii = zip(kk, vv)
+        kk = m.group(1).split(b",")
+        vv = m.group(2).split(b",")
+        ii = list(zip(kk, vv))
         ii.sort()
 
         self.assertEqual(len(ii), len(o))
@@ -196,8 +197,6 @@ class HstoreTestCase(ConnectingTestCase):
             self.assertQuotedEqual(ii[3][1], b"'" + encc + b"'")
 
     def test_parse(self):
-        from psycopg2.extras import HstoreAdapter
-
         def ok(s, d):
             self.assertEqual(HstoreAdapter.parse(s, None), d)
 
@@ -227,8 +226,6 @@ class HstoreTestCase(ConnectingTestCase):
 
     @skip_if_no_hstore
     def test_register_conn(self):
-        from psycopg2.extras import register_hstore
-
         register_hstore(self.conn)
         cur = self.conn.cursor()
         cur.execute("select null::hstore, ''::hstore, 'a => b'::hstore")
@@ -239,8 +236,6 @@ class HstoreTestCase(ConnectingTestCase):
 
     @skip_if_no_hstore
     def test_register_curs(self):
-        from psycopg2.extras import register_hstore
-
         cur = self.conn.cursor()
         register_hstore(cur)
         cur.execute("select null::hstore, ''::hstore, 'a => b'::hstore")
@@ -250,9 +245,8 @@ class HstoreTestCase(ConnectingTestCase):
         self.assertEqual(t[2], {'a': 'b'})
 
     @skip_if_no_hstore
+    @skip_from_python(3)
     def test_register_unicode(self):
-        from psycopg2.extras import register_hstore
-
         register_hstore(self.conn, unicode=True)
         cur = self.conn.cursor()
         cur.execute("select null::hstore, ''::hstore, 'a => b'::hstore")
@@ -265,8 +259,6 @@ class HstoreTestCase(ConnectingTestCase):
 
     @skip_if_no_hstore
     def test_register_globally(self):
-        from psycopg2.extras import register_hstore, HstoreAdapter
-
         oids = HstoreAdapter.get_oids(self.conn)
         try:
             register_hstore(self.conn, globally=True)
@@ -289,7 +281,6 @@ class HstoreTestCase(ConnectingTestCase):
 
     @skip_if_no_hstore
     def test_roundtrip(self):
-        from psycopg2.extras import register_hstore
         register_hstore(self.conn)
         cur = self.conn.cursor()
 
@@ -304,22 +295,22 @@ class HstoreTestCase(ConnectingTestCase):
         ok({})
         ok({'a': 'b', 'c': None})
 
-        ab = map(chr, range(32, 128))
+        ab = list(map(chr, range(32, 128)))
         ok(dict(zip(ab, ab)))
         ok({''.join(ab): ''.join(ab)})
 
         self.conn.set_client_encoding('latin1')
-        if sys.version_info[0] < 3:
+        if PY2:
             ab = map(chr, range(32, 127) + range(160, 255))
         else:
-            ab = bytes(range(32, 127) + range(160, 255)).decode('latin1')
+            ab = bytes(list(range(32, 127)) + list(range(160, 255))).decode('latin1')
 
         ok({''.join(ab): ''.join(ab)})
         ok(dict(zip(ab, ab)))
 
     @skip_if_no_hstore
+    @skip_from_python(3)
     def test_roundtrip_unicode(self):
-        from psycopg2.extras import register_hstore
         register_hstore(self.conn, unicode=True)
         cur = self.conn.cursor()
 
@@ -348,7 +339,6 @@ class HstoreTestCase(ConnectingTestCase):
 
         # Note: None as conn_or_cursor is just for testing: not public
         # interface and it may break in future.
-        from psycopg2.extras import register_hstore
         register_hstore(None, globally=True, oid=oid)
         try:
             cur.execute("select null::hstore, ''::hstore, 'a => b'::hstore")
@@ -363,22 +353,19 @@ class HstoreTestCase(ConnectingTestCase):
     @skip_if_no_hstore
     @skip_before_postgres(8, 3)
     def test_roundtrip_array(self):
-        from psycopg2.extras import register_hstore
         register_hstore(self.conn)
 
-        ds = []
-        ds.append({})
-        ds.append({'a': 'b', 'c': None})
+        ds = [{}, {'a': 'b', 'c': None}]
 
-        ab = map(chr, range(32, 128))
+        ab = list(map(chr, range(32, 128)))
         ds.append(dict(zip(ab, ab)))
         ds.append({''.join(ab): ''.join(ab)})
 
         self.conn.set_client_encoding('latin1')
-        if sys.version_info[0] < 3:
+        if PY2:
             ab = map(chr, range(32, 127) + range(160, 255))
         else:
-            ab = bytes(range(32, 127) + range(160, 255)).decode('latin1')
+            ab = bytes(list(range(32, 127)) + list(range(160, 255))).decode('latin1')
 
         ds.append({''.join(ab): ''.join(ab)})
         ds.append(dict(zip(ab, ab)))
@@ -391,7 +378,6 @@ class HstoreTestCase(ConnectingTestCase):
     @skip_if_no_hstore
     @skip_before_postgres(8, 3)
     def test_array_cast(self):
-        from psycopg2.extras import register_hstore
         register_hstore(self.conn)
         cur = self.conn.cursor()
         cur.execute("select array['a=>1'::hstore, 'b=>2'::hstore];")
@@ -404,7 +390,6 @@ class HstoreTestCase(ConnectingTestCase):
         cur.execute("select 'hstore'::regtype::oid, 'hstore[]'::regtype::oid")
         oid, aoid = cur.fetchone()
 
-        from psycopg2.extras import register_hstore
         register_hstore(None, globally=True, oid=oid, array_oid=aoid)
         try:
             cur.execute("""
@@ -422,9 +407,6 @@ class HstoreTestCase(ConnectingTestCase):
 
     @skip_if_no_hstore
     def test_non_dbapi_connection(self):
-        from psycopg2.extras import RealDictConnection
-        from psycopg2.extras import register_hstore
-
         conn = self.connect(connection_factory=RealDictConnection)
         try:
             register_hstore(conn)
@@ -447,10 +429,10 @@ class HstoreTestCase(ConnectingTestCase):
 def skip_if_no_composite(f):
     @wraps(f)
     def skip_if_no_composite_(self):
-        if self.conn.server_version < 80000:
+        if self.conn.info.server_version < 80000:
             return self.skipTest(
                 "server version %s doesn't support composite types"
-                % self.conn.server_version)
+                % self.conn.info.server_version)
 
         return f(self)
 
@@ -491,8 +473,6 @@ class AdaptTypeTestCase(ConnectingTestCase):
             ext.register_adapter(type(None), orig_adapter)
 
     def test_tokenization(self):
-        from psycopg2.extras import CompositeCaster
-
         def ok(s, v):
             self.assertEqual(CompositeCaster.tokenize(s), v)
 
@@ -513,7 +493,7 @@ class AdaptTypeTestCase(ConnectingTestCase):
            '@,A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V,W,X,Y,Z,[,"\\\\",],'
            '^,_,`,a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z,{,|,},'
            '~,\x7f)',
-           map(chr, range(1, 128)))
+           list(map(chr, range(1, 128))))
         ok('(,"\x01\x02\x03\x04\x05\x06\x07\x08\t\n\x0b\x0c\r\x0e\x0f'
            '\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f !'
            '""#$%&\'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\\\]'
@@ -541,16 +521,10 @@ class AdaptTypeTestCase(ConnectingTestCase):
         self.assertEqual(v[0], 10)
         self.assertEqual(v[1], "hello")
         self.assertEqual(v[2], date(2011, 1, 2))
-
-        try:
-            from collections import namedtuple          # noqa
-        except ImportError:
-            pass
-        else:
-            self.assert_(t.type is not tuple)
-            self.assertEqual(v.anint, 10)
-            self.assertEqual(v.astring, "hello")
-            self.assertEqual(v.adate, date(2011, 1, 2))
+        self.assert_(t.type is not tuple)
+        self.assertEqual(v.anint, 10)
+        self.assertEqual(v.astring, "hello")
+        self.assertEqual(v.adate, date(2011, 1, 2))
 
     @skip_if_no_composite
     def test_empty_string(self):
@@ -591,13 +565,7 @@ class AdaptTypeTestCase(ConnectingTestCase):
         v = curs.fetchone()[0]
 
         self.assertEqual(r, v)
-
-        try:
-            from collections import namedtuple              # noqa
-        except ImportError:
-            pass
-        else:
-            self.assertEqual(v.anotherpair.apair.astring, "hello")
+        self.assertEqual(v.anotherpair.apair.astring, "hello")
 
     @skip_if_no_composite
     def test_register_on_cursor(self):
@@ -701,7 +669,6 @@ class AdaptTypeTestCase(ConnectingTestCase):
     @skip_if_no_composite
     def test_wrong_schema(self):
         oid = self._create_type("type_ii", [("a", "integer"), ("b", "integer")])
-        from psycopg2.extras import CompositeCaster
         c = CompositeCaster('type_ii', oid, [('a', 23), ('b', 23), ('c', 23)])
         curs = self.conn.cursor()
         psycopg2.extensions.register_type(c.typecaster, curs)
@@ -756,8 +723,6 @@ class AdaptTypeTestCase(ConnectingTestCase):
 
     @skip_if_no_composite
     def test_non_dbapi_connection(self):
-        from psycopg2.extras import RealDictConnection
-        from psycopg2.extras import register_composite
         self._create_type("type_ii", [("a", "integer"), ("b", "integer")])
 
         conn = self.connect(connection_factory=RealDictConnection)
@@ -783,8 +748,6 @@ class AdaptTypeTestCase(ConnectingTestCase):
         oid = self._create_type("type_isd",
             [('anint', 'integer'), ('astring', 'text'), ('adate', 'date')])
 
-        from psycopg2.extras import register_composite, CompositeCaster
-
         class DictComposite(CompositeCaster):
             def make(self, values):
                 return dict(zip(self.attnames, values))
@@ -827,30 +790,6 @@ class AdaptTypeTestCase(ConnectingTestCase):
         return oid
 
 
-def skip_if_json_module(f):
-    """Skip a test if a Python json module *is* available"""
-    @wraps(f)
-    def skip_if_json_module_(self):
-        if psycopg2.extras.json is not None:
-            return self.skipTest("json module is available")
-
-        return f(self)
-
-    return skip_if_json_module_
-
-
-def skip_if_no_json_module(f):
-    """Skip a test if no Python json module is available"""
-    @wraps(f)
-    def skip_if_no_json_module_(self):
-        if psycopg2.extras.json is None:
-            return self.skipTest("json module not available")
-
-        return f(self)
-
-    return skip_if_no_json_module_
-
-
 def skip_if_no_json_type(f):
     """Skip a test if PostgreSQL json type is not available"""
     @wraps(f)
@@ -866,26 +805,7 @@ def skip_if_no_json_type(f):
 
 
 class JsonTestCase(ConnectingTestCase):
-    @skip_if_json_module
-    def test_module_not_available(self):
-        from psycopg2.extras import Json
-        self.assertRaises(ImportError, Json(None).getquoted)
-
-    @skip_if_json_module
-    def test_customizable_with_module_not_available(self):
-        from psycopg2.extras import Json
-
-        class MyJson(Json):
-            def dumps(self, obj):
-                assert obj is None
-                return "hi"
-
-        self.assertEqual(MyJson(None).getquoted(), "'hi'")
-
-    @skip_if_no_json_module
     def test_adapt(self):
-        from psycopg2.extras import json, Json
-
         objs = [None, "te'xt", 123, 123.45,
             u'\xe0\u20ac', ['a', 100], {'a': 100}]
 
@@ -894,10 +814,7 @@ class JsonTestCase(ConnectingTestCase):
             self.assertQuotedEqual(curs.mogrify("%s", (Json(obj),)),
                 psycopg2.extensions.QuotedString(json.dumps(obj)).getquoted())
 
-    @skip_if_no_json_module
     def test_adapt_dumps(self):
-        from psycopg2.extras import json, Json
-
         class DecimalEncoder(json.JSONEncoder):
             def default(self, obj):
                 if isinstance(obj, Decimal):
@@ -912,10 +829,7 @@ class JsonTestCase(ConnectingTestCase):
         self.assertQuotedEqual(curs.mogrify("%s", (Json(obj, dumps=dumps),)),
             b"'123.45'")
 
-    @skip_if_no_json_module
     def test_adapt_subclass(self):
-        from psycopg2.extras import json, Json
-
         class DecimalEncoder(json.JSONEncoder):
             def default(self, obj):
                 if isinstance(obj, Decimal):
@@ -930,9 +844,7 @@ class JsonTestCase(ConnectingTestCase):
         obj = Decimal('123.45')
         self.assertQuotedEqual(curs.mogrify("%s", (MyJson(obj),)), b"'123.45'")
 
-    @skip_if_no_json_module
     def test_register_on_dict(self):
-        from psycopg2.extras import Json
         psycopg2.extensions.register_adapter(dict, Json)
 
         try:
@@ -952,7 +864,6 @@ class JsonTestCase(ConnectingTestCase):
         self.assertRaises(psycopg2.ProgrammingError,
             psycopg2.extras.register_json, self.conn)
 
-    @skip_if_no_json_module
     @skip_before_postgres(9, 2)
     def test_default_cast(self):
         curs = self.conn.cursor()
@@ -963,7 +874,6 @@ class JsonTestCase(ConnectingTestCase):
         curs.execute("""select array['{"a": 100.0, "b": null}']::json[]""")
         self.assertEqual(curs.fetchone()[0], [{'a': 100.0, 'b': None}])
 
-    @skip_if_no_json_module
     @skip_if_no_json_type
     def test_register_on_connection(self):
         psycopg2.extras.register_json(self.conn)
@@ -971,7 +881,6 @@ class JsonTestCase(ConnectingTestCase):
         curs.execute("""select '{"a": 100.0, "b": null}'::json""")
         self.assertEqual(curs.fetchone()[0], {'a': 100.0, 'b': None})
 
-    @skip_if_no_json_module
     @skip_if_no_json_type
     def test_register_on_cursor(self):
         curs = self.conn.cursor()
@@ -979,7 +888,6 @@ class JsonTestCase(ConnectingTestCase):
         curs.execute("""select '{"a": 100.0, "b": null}'::json""")
         self.assertEqual(curs.fetchone()[0], {'a': 100.0, 'b': None})
 
-    @skip_if_no_json_module
     @skip_if_no_json_type
     def test_register_globally(self):
         old = psycopg2.extensions.string_types.get(114)
@@ -997,7 +905,6 @@ class JsonTestCase(ConnectingTestCase):
             if olda:
                 psycopg2.extensions.register_type(olda)
 
-    @skip_if_no_json_module
     @skip_if_no_json_type
     def test_loads(self):
         json = psycopg2.extras.json
@@ -1011,10 +918,8 @@ class JsonTestCase(ConnectingTestCase):
         self.assert_(isinstance(data['a'], Decimal))
         self.assertEqual(data['a'], Decimal('100.0'))
 
-    @skip_if_no_json_module
     @skip_if_no_json_type
     def test_no_conn_curs(self):
-        from psycopg2._json import _get_json_oids
         oid, array_oid = _get_json_oids(self.conn)
 
         old = psycopg2.extensions.string_types.get(114)
@@ -1038,7 +943,6 @@ class JsonTestCase(ConnectingTestCase):
             if olda:
                 psycopg2.extensions.register_type(olda)
 
-    @skip_if_no_json_module
     @skip_before_postgres(9, 2)
     def test_register_default(self):
         curs = self.conn.cursor()
@@ -1057,7 +961,6 @@ class JsonTestCase(ConnectingTestCase):
         self.assert_(isinstance(data[0]['a'], Decimal))
         self.assertEqual(data[0]['a'], Decimal('100.0'))
 
-    @skip_if_no_json_module
     @skip_if_no_json_type
     def test_null(self):
         psycopg2.extras.register_json(self.conn)
@@ -1067,7 +970,6 @@ class JsonTestCase(ConnectingTestCase):
         curs.execute("""select NULL::json[]""")
         self.assertEqual(curs.fetchone()[0], None)
 
-    @skip_if_no_json_module
     def test_no_array_oid(self):
         curs = self.conn.cursor()
         t1, t2 = psycopg2.extras.register_json(curs, oid=25)
@@ -1079,7 +981,6 @@ class JsonTestCase(ConnectingTestCase):
         self.assertEqual(data['a'], 100)
         self.assertEqual(data['b'], None)
 
-    @skip_if_no_json_module
     def test_str(self):
         snowman = u"\u2603"
         obj = {'a': [1, 2, snowman]}
@@ -1090,7 +991,6 @@ class JsonTestCase(ConnectingTestCase):
         self.assert_(s.startswith("'"))
         self.assert_(s.endswith("'"))
 
-    @skip_if_no_json_module
     @skip_before_postgres(8, 2)
     def test_scs(self):
         cnn_on = self.connect(options="-c standard_conforming_strings=on")
@@ -1114,10 +1014,10 @@ def skip_if_no_jsonb_type(f):
     return skip_before_postgres(9, 4)(f)
 
 
+@skip_if_no_jsonb_type
 class JsonbTestCase(ConnectingTestCase):
     @staticmethod
     def myloads(s):
-        import json
         rv = json.loads(s)
         rv['test'] = 1
         return rv
@@ -1203,13 +1103,9 @@ class JsonbTestCase(ConnectingTestCase):
         curs.execute("""select NULL::jsonb[]""")
         self.assertEqual(curs.fetchone()[0], None)
 
-decorate_all_tests(JsonbTestCase, skip_if_no_json_module)
-decorate_all_tests(JsonbTestCase, skip_if_no_jsonb_type)
-
 
 class RangeTestCase(unittest.TestCase):
     def test_noparam(self):
-        from psycopg2.extras import Range
         r = Range()
 
         self.assert_(not r.isempty)
@@ -1221,7 +1117,6 @@ class RangeTestCase(unittest.TestCase):
         self.assert_(not r.upper_inc)
 
     def test_empty(self):
-        from psycopg2.extras import Range
         r = Range(empty=True)
 
         self.assert_(r.isempty)
@@ -1233,7 +1128,6 @@ class RangeTestCase(unittest.TestCase):
         self.assert_(not r.upper_inc)
 
     def test_nobounds(self):
-        from psycopg2.extras import Range
         r = Range(10, 20)
         self.assertEqual(r.lower, 10)
         self.assertEqual(r.upper, 20)
@@ -1244,7 +1138,6 @@ class RangeTestCase(unittest.TestCase):
         self.assert_(not r.upper_inc)
 
     def test_bounds(self):
-        from psycopg2.extras import Range
         for bounds, lower_inc, upper_inc in [
                 ('[)', True, False),
                 ('(]', False, True),
@@ -1260,7 +1153,6 @@ class RangeTestCase(unittest.TestCase):
             self.assertEqual(r.upper_inc, upper_inc)
 
     def test_keywords(self):
-        from psycopg2.extras import Range
         r = Range(upper=20)
         self.assertEqual(r.lower, None)
         self.assertEqual(r.upper, 20)
@@ -1280,12 +1172,10 @@ class RangeTestCase(unittest.TestCase):
         self.assert_(not r.upper_inc)
 
     def test_bad_bounds(self):
-        from psycopg2.extras import Range
         self.assertRaises(ValueError, Range, bounds='(')
         self.assertRaises(ValueError, Range, bounds='[}')
 
     def test_in(self):
-        from psycopg2.extras import Range
         r = Range(empty=True)
         self.assert_(10 not in r)
 
@@ -1337,7 +1227,6 @@ class RangeTestCase(unittest.TestCase):
         self.assert_(21 not in r)
 
     def test_nonzero(self):
-        from psycopg2.extras import Range
         self.assert_(Range())
         self.assert_(Range(10, 20))
         self.assert_(not Range(empty=True))
@@ -1347,7 +1236,6 @@ class RangeTestCase(unittest.TestCase):
             self.assert_(r1 == r2)
             self.assert_(hash(r1) == hash(r2))
 
-        from psycopg2.extras import Range
         assert_equal(Range(empty=True), Range(empty=True))
         assert_equal(Range(), Range())
         assert_equal(Range(10, None), Range(10, None))
@@ -1364,12 +1252,9 @@ class RangeTestCase(unittest.TestCase):
         assert_not_equal(Range(10, 20, '[)'), Range(10, 20, '[]'))
 
     def test_eq_wrong_type(self):
-        from psycopg2.extras import Range
         self.assertNotEqual(Range(10, 20), ())
 
     def test_eq_subclass(self):
-        from psycopg2.extras import Range, NumericRange
-
         class IntRange(NumericRange):
             pass
 
@@ -1384,7 +1269,6 @@ class RangeTestCase(unittest.TestCase):
     # and consistent.
 
     def test_lt_ordering(self):
-        from psycopg2.extras import Range
         self.assert_(Range(empty=True) < Range(0, 4))
         self.assert_(not Range(1, 2) < Range(0, 4))
         self.assert_(Range(0, 4) < Range(1, 2))
@@ -1400,7 +1284,6 @@ class RangeTestCase(unittest.TestCase):
             self.assert_(not Range(1, 2) < 1)
 
     def test_gt_ordering(self):
-        from psycopg2.extras import Range
         self.assert_(not Range(empty=True) > Range(0, 4))
         self.assert_(Range(1, 2) > Range(0, 4))
         self.assert_(not Range(0, 4) > Range(1, 2))
@@ -1416,7 +1299,6 @@ class RangeTestCase(unittest.TestCase):
             self.assert_(Range(1, 2) > 1)
 
     def test_le_ordering(self):
-        from psycopg2.extras import Range
         self.assert_(Range(empty=True) <= Range(0, 4))
         self.assert_(not Range(1, 2) <= Range(0, 4))
         self.assert_(Range(0, 4) <= Range(1, 2))
@@ -1432,7 +1314,6 @@ class RangeTestCase(unittest.TestCase):
             self.assert_(not Range(1, 2) <= 1)
 
     def test_ge_ordering(self):
-        from psycopg2.extras import Range
         self.assert_(not Range(empty=True) >= Range(0, 4))
         self.assert_(Range(1, 2) >= Range(0, 4))
         self.assert_(not Range(0, 4) >= Range(1, 2))
@@ -1448,25 +1329,50 @@ class RangeTestCase(unittest.TestCase):
             self.assert_(Range(1, 2) >= 1)
 
     def test_pickling(self):
-        from psycopg2.extras import Range
-
         r = Range(0, 4)
         self.assertEqual(loads(dumps(r)), r)
 
+    def test_str(self):
+        '''
+        Range types should have a short and readable ``str`` implementation.
 
-def skip_if_no_range(f):
-    @wraps(f)
-    def skip_if_no_range_(self):
-        if self.conn.server_version < 90200:
-            return self.skipTest(
-                "server version %s doesn't support range types"
-                % self.conn.server_version)
+        Using ``repr`` for all string conversions can be very unreadable for
+        longer types like ``DateTimeTZRange``.
+        '''
 
-        return f(self)
+        # Using the "u" prefix to make sure we have the proper return types in
+        # Python2
+        expected = [
+            u'(0, 4)',
+            u'[0, 4]',
+            u'(0, 4]',
+            u'[0, 4)',
+            u'empty',
+        ]
+        results = []
 
-    return skip_if_no_range_
+        for bounds in ('()', '[]', '(]', '[)'):
+            r = Range(0, 4, bounds=bounds)
+            results.append(text_type(r))
+
+        r = Range(empty=True)
+        results.append(text_type(r))
+        self.assertEqual(results, expected)
+
+    def test_str_datetime(self):
+        '''
+        Date-Time ranges should return a human-readable string as well on
+        string conversion.
+        '''
+        tz = FixedOffsetTimezone(-5 * 60, "EST")
+        r = DateTimeTZRange(datetime(2010, 1, 1, tzinfo=tz),
+                            datetime(2011, 1, 1, tzinfo=tz))
+        expected = u'[2010-01-01 00:00:00-05:00, 2011-01-01 00:00:00-05:00)'
+        result = text_type(r)
+        self.assertEqual(result, expected)
 
 
+@skip_before_postgres(9, 2, "range not supported before postgres 9.2")
 class RangeCasterTestCase(ConnectingTestCase):
 
     builtin_ranges = ('int4range', 'int8range', 'numrange',
@@ -1480,7 +1386,6 @@ class RangeCasterTestCase(ConnectingTestCase):
             self.assertEqual(r, None)
 
     def test_cast_empty(self):
-        from psycopg2.extras import Range
         cur = self.conn.cursor()
         for type in self.builtin_ranges:
             cur.execute("select 'empty'::%s" % type)
@@ -1489,7 +1394,6 @@ class RangeCasterTestCase(ConnectingTestCase):
             self.assert_(r.isempty)
 
     def test_cast_inf(self):
-        from psycopg2.extras import Range
         cur = self.conn.cursor()
         for type in self.builtin_ranges:
             cur.execute("select '(,)'::%s" % type)
@@ -1500,7 +1404,6 @@ class RangeCasterTestCase(ConnectingTestCase):
             self.assert_(r.upper_inf)
 
     def test_cast_numbers(self):
-        from psycopg2.extras import NumericRange
         cur = self.conn.cursor()
         for type in ('int4range', 'int8range'):
             cur.execute("select '(10,20)'::%s" % type)
@@ -1526,7 +1429,6 @@ class RangeCasterTestCase(ConnectingTestCase):
         self.assert_(not r.upper_inc)
 
     def test_cast_date(self):
-        from psycopg2.extras import DateRange
         cur = self.conn.cursor()
         cur.execute("select '(2000-01-01,2012-12-31)'::daterange")
         r = cur.fetchone()[0]
@@ -1540,7 +1442,6 @@ class RangeCasterTestCase(ConnectingTestCase):
         self.assert_(not r.upper_inc)
 
     def test_cast_timestamp(self):
-        from psycopg2.extras import DateTimeRange
         cur = self.conn.cursor()
         ts1 = datetime(2000, 1, 1)
         ts2 = datetime(2000, 12, 31, 23, 59, 59, 999)
@@ -1556,8 +1457,6 @@ class RangeCasterTestCase(ConnectingTestCase):
         self.assert_(not r.upper_inc)
 
     def test_cast_timestamptz(self):
-        from psycopg2.extras import DateTimeTZRange
-        from psycopg2.tz import FixedOffsetTimezone
         cur = self.conn.cursor()
         ts1 = datetime(2000, 1, 1, tzinfo=FixedOffsetTimezone(600))
         ts2 = datetime(2000, 12, 31, 23, 59, 59, 999,
@@ -1574,7 +1473,6 @@ class RangeCasterTestCase(ConnectingTestCase):
         self.assert_(r.upper_inc)
 
     def test_adapt_number_range(self):
-        from psycopg2.extras import NumericRange
         cur = self.conn.cursor()
 
         r = NumericRange(empty=True)
@@ -1602,7 +1500,6 @@ class RangeCasterTestCase(ConnectingTestCase):
         self.assert_(r1.upper_inc)
 
     def test_adapt_numeric_range(self):
-        from psycopg2.extras import NumericRange
         cur = self.conn.cursor()
 
         r = NumericRange(empty=True)
@@ -1630,11 +1527,9 @@ class RangeCasterTestCase(ConnectingTestCase):
         self.assert_(r1.upper_inc)
 
     def test_adapt_date_range(self):
-        from psycopg2.extras import DateRange, DateTimeRange, DateTimeTZRange
-        from psycopg2.tz import FixedOffsetTimezone
         cur = self.conn.cursor()
 
-        d1 = date(2012, 01, 01)
+        d1 = date(2012, 1, 1)
         d2 = date(2012, 12, 31)
         r = DateRange(d1, d2)
         cur.execute("select %s", (r,))
@@ -1664,7 +1559,6 @@ class RangeCasterTestCase(ConnectingTestCase):
         self.assert_(r1.upper_inc)
 
     def test_register_range_adapter(self):
-        from psycopg2.extras import Range, register_range
         cur = self.conn.cursor()
         cur.execute("create type textrange as range (subtype=text)")
         rc = register_range('textrange', 'TextRange', cur)
@@ -1694,7 +1588,6 @@ class RangeCasterTestCase(ConnectingTestCase):
         del ext.adapters[rc.range, ext.ISQLQuote]
 
     def test_range_escaping(self):
-        from psycopg2.extras import register_range
         cur = self.conn.cursor()
         cur.execute("create type textrange as range (subtype=text)")
         rc = register_range('textrange', 'TextRange', cur)
@@ -1708,8 +1601,8 @@ class RangeCasterTestCase(ConnectingTestCase):
         bounds = ['[)', '(]', '()', '[]']
         ranges = [TextRange(low, up, bounds[i % 4])
             for i, (low, up) in enumerate(zip(
-                [None] + map(chr, range(1, 128)),
-                map(chr, range(1, 128)) + [None],
+                [None] + list(map(chr, range(1, 128))),
+                list(map(chr, range(1, 128))) + [None],
             ))]
         ranges.append(TextRange())
         ranges.append(TextRange(empty=True))
@@ -1748,7 +1641,6 @@ class RangeCasterTestCase(ConnectingTestCase):
         del ext.adapters[TextRange, ext.ISQLQuote]
 
     def test_range_not_found(self):
-        from psycopg2.extras import register_range
         cur = self.conn.cursor()
         self.assertRaises(psycopg2.ProgrammingError,
             register_range, 'nosuchrange', 'FailRange', cur)
@@ -1762,7 +1654,6 @@ class RangeCasterTestCase(ConnectingTestCase):
         cur.execute("create type rs.r3 as range (subtype=text)")
         cur.execute("savepoint x")
 
-        from psycopg2.extras import register_range
         ra1 = register_range('r1', 'r1', cur)
         ra2 = register_range('r2', 'r2', cur)
         rars2 = register_range('rs.r2', 'r2', cur)
@@ -1784,8 +1675,6 @@ class RangeCasterTestCase(ConnectingTestCase):
         for r in [ra1, ra2, rars2, rars3]:
             del ext.adapters[r.range, ext.ISQLQuote]
 
-decorate_all_tests(RangeCasterTestCase, skip_if_no_range)
-
 
 def test_suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
