@@ -1,7 +1,7 @@
 # testutils.py - utility module for psycopg2 testing.
 
 #
-# Copyright (C) 2010-2011 Daniele Varrazzo  <daniele.varrazzo@gmail.com>
+# Copyright (C) 2010-2019 Daniele Varrazzo  <daniele.varrazzo@gmail.com>
 #
 # psycopg2 is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published
@@ -25,48 +25,43 @@
 import re
 import os
 import sys
+import types
+import ctypes
 import select
 import platform
 import unittest
 from functools import wraps
-from testconfig import dsn, repl_dsn
+from ctypes.util import find_library
 
+import psycopg2
+import psycopg2.errors
+import psycopg2.extensions
+from psycopg2.compat import PY2, PY3, text_type
 
-if hasattr(unittest, 'skipIf'):
-    skip = unittest.skip
-    skipIf = unittest.skipIf
+from .testconfig import green, dsn, repl_dsn
+
+# Python 2/3 compatibility
+
+if PY2:
+    # Python 2
+    from StringIO import StringIO
+    TextIOBase = object
+    long = long
+    reload = reload
+    unichr = unichr
 
 else:
-    import warnings
+    # Python 3
+    from io import StringIO         # noqa
+    from io import TextIOBase       # noqa
+    from importlib import reload    # noqa
+    long = int
+    unichr = chr
 
-    def skipIf(cond, msg):
-        def skipIf_(f):
-            @wraps(f)
-            def skipIf__(self):
-                if cond:
-                    with warnings.catch_warnings():
-                        warnings.simplefilter('always', UserWarning)
-                        warnings.warn(msg)
-                    return
-                else:
-                    return f(self)
-            return skipIf__
-        return skipIf_
-
-    def skip(msg):
-        return skipIf(True, msg)
-
-    def skipTest(self, msg):
-        with warnings.catch_warnings():
-            warnings.simplefilter('always', UserWarning)
-            warnings.warn(msg)
-        return
-
-    unittest.TestCase.skipTest = skipTest
 
 # Silence warnings caused by the stubbornness of the Python unittest
 # maintainers
-# http://bugs.python.org/issue9424
+# https://bugs.python.org/issue9424
 if (not hasattr(unittest.TestCase, 'assert_')
         or unittest.TestCase.assert_ is not unittest.TestCase.assertTrue):
     # mavaff...
@@ -80,6 +75,7 @@ def assertDsnEqual(self, dsn1, dsn2, msg=None):
     """Check that two conninfo string have the same content"""
     self.assertEqual(set(dsn1.split()), set(dsn2.split()), msg)
 
+
 unittest.TestCase.assertDsnEqual = assertDsnEqual
 
 
@@ -104,7 +100,7 @@ class ConnectingTestCase(unittest.TestCase):
     def assertQuotedEqual(self, first, second, msg=None):
         """Compare two quoted strings disregarding eventual E'' quotes"""
         def f(s):
-            if isinstance(s, unicode):
+            if isinstance(s, text_type):
                 return re.sub(r"\bE'", "'", s)
             elif isinstance(first, bytes):
                 return re.sub(br"\bE'", b"'", s)
@@ -125,7 +121,6 @@ class ConnectingTestCase(unittest.TestCase):
             conninfo = kwargs.pop('dsn')
         else:
             conninfo = dsn
-        import psycopg2
         conn = psycopg2.connect(conninfo, **kwargs)
         self._conns.append(conn)
         return conn
@@ -144,7 +139,6 @@ class ConnectingTestCase(unittest.TestCase):
 
         if 'dsn' not in kwargs:
             kwargs['dsn'] = repl_dsn
-        import psycopg2
         try:
             conn = self.connect(**kwargs)
             if conn.async_ == 1:
@@ -173,7 +167,6 @@ class ConnectingTestCase(unittest.TestCase):
 
     # for use with async connections only
     def wait(self, cur_or_conn):
-        import psycopg2.extensions
         pollable = cur_or_conn
         if not hasattr(pollable, 'poll'):
             pollable = cur_or_conn.connection
@@ -182,32 +175,57 @@ class ConnectingTestCase(unittest.TestCase):
             if state == psycopg2.extensions.POLL_OK:
                 break
             elif state == psycopg2.extensions.POLL_READ:
-                select.select([pollable], [], [], 10)
+                select.select([pollable], [], [], 1)
             elif state == psycopg2.extensions.POLL_WRITE:
-                select.select([], [pollable], [], 10)
+                select.select([], [pollable], [], 1)
             else:
                 raise Exception("Unexpected result from poll: %r", state)
 
+    _libpq = None
 
-def decorate_all_tests(cls, *decorators):
+    @property
+    def libpq(self):
+        """Return a ctypes wrapper for the libpq library"""
+        if ConnectingTestCase._libpq is not None:
+            return ConnectingTestCase._libpq
+
+        libname = find_library('pq')
+        if libname is None and platform.system() == 'Windows':
+            raise self.skipTest("can't import libpq on windows")
+
+        rv = ConnectingTestCase._libpq = ctypes.pydll.LoadLibrary(libname)
+        return rv
+
+
+def decorate_all_tests(obj, *decorators):
     """
-    Apply all the *decorators* to all the tests defined in the TestCase *cls*.
+    Apply all the *decorators* to all the tests defined in the TestCase *obj*.
+
+    The decorator can also be applied to a decorator: if *obj* is a function,
+    return a new decorator which can be applied either to a method or to a
+    class, in which case it will decorate all the tests.
     """
-    for n in dir(cls):
+    if isinstance(obj, types.FunctionType):
+        def decorator(func_or_cls):
+            if isinstance(func_or_cls, types.FunctionType):
+                return obj(func_or_cls)
+            else:
+                decorate_all_tests(func_or_cls, obj)
+                return func_or_cls
+
+        return decorator
+
+    for n in dir(obj):
         if n.startswith('test'):
             for d in decorators:
-                setattr(cls, n, d(getattr(cls, n)))
+                setattr(obj, n, d(getattr(obj, n)))
 
 
+@decorate_all_tests
 def skip_if_no_uuid(f):
-    """Decorator to skip a test if uuid is not supported by Py/PG."""
+    """Decorator to skip a test if uuid is not supported by PG."""
     @wraps(f)
     def skip_if_no_uuid_(self):
-        try:
-            import uuid             # noqa
-        except ImportError:
-            return self.skipTest("uuid not available in this Python version")
-
         try:
             cur = self.conn.cursor()
             cur.execute("select typname from pg_type where typname = 'uuid'")
@@ -223,16 +241,16 @@ def skip_if_no_uuid(f):
     return skip_if_no_uuid_
 
 
+@decorate_all_tests
 def skip_if_tpc_disabled(f):
     """Skip a test if the server has tpc support disabled."""
     @wraps(f)
     def skip_if_tpc_disabled_(self):
-        from psycopg2 import ProgrammingError
         cnn = self.connect()
         cur = cnn.cursor()
         try:
             cur.execute("SHOW max_prepared_transactions;")
-        except ProgrammingError:
+        except psycopg2.ProgrammingError:
             return self.skipTest(
                 "server too old: two phase transactions not supported.")
         else:
@@ -248,43 +266,22 @@ def skip_if_tpc_disabled(f):
     return skip_if_tpc_disabled_
 
 
-def skip_if_no_namedtuple(f):
-    @wraps(f)
-    def skip_if_no_namedtuple_(self):
-        try:
-            from collections import namedtuple              # noqa
-        except ImportError:
-            return self.skipTest("collections.namedtuple not available")
-        else:
-            return f(self)
-
-    return skip_if_no_namedtuple_
-
-
-def skip_if_no_iobase(f):
-    """Skip a test if io.TextIOBase is not available."""
-    @wraps(f)
-    def skip_if_no_iobase_(self):
-        try:
-            from io import TextIOBase                       # noqa
-        except ImportError:
-            return self.skipTest("io.TextIOBase not found.")
-        else:
-            return f(self)
-
-    return skip_if_no_iobase_
-
-
 def skip_before_postgres(*ver):
     """Skip a test on PostgreSQL before a certain version."""
+    reason = None
+    if isinstance(ver[-1], str):
+        ver, reason = ver[:-1], ver[-1]
+
     ver = ver + (0,) * (3 - len(ver))
 
+    @decorate_all_tests
     def skip_before_postgres_(f):
         @wraps(f)
         def skip_before_postgres__(self):
-            if self.conn.server_version < int("%d%02d%02d" % ver):
-                return self.skipTest("skipped because PostgreSQL %s"
-                    % self.conn.server_version)
+            if self.conn.info.server_version < int("%d%02d%02d" % ver):
+                return self.skipTest(
+                    reason or "skipped because PostgreSQL %s"
+                    % self.conn.info.server_version)
             else:
                 return f(self)
 
@@ -296,12 +293,13 @@ def skip_after_postgres(*ver):
     """Skip a test on PostgreSQL after (including) a certain version."""
     ver = ver + (0,) * (3 - len(ver))
 
+    @decorate_all_tests
     def skip_after_postgres_(f):
         @wraps(f)
         def skip_after_postgres__(self):
-            if self.conn.server_version >= int("%d%02d%02d" % ver):
+            if self.conn.info.server_version >= int("%d%02d%02d" % ver):
                 return self.skipTest("skipped because PostgreSQL %s"
-                    % self.conn.server_version)
+                    % self.conn.info.server_version)
             else:
                 return f(self)
 
@@ -310,7 +308,6 @@ def skip_after_postgres(*ver):
 
 
 def libpq_version():
-    import psycopg2
     v = psycopg2.__libpq_version__
     if v >= 90100:
         v = min(v, psycopg2.extensions.libpq_version())
@@ -321,16 +318,13 @@ def skip_before_libpq(*ver):
     """Skip a test if libpq we're linked to is older than a certain version."""
     ver = ver + (0,) * (3 - len(ver))
 
-    def skip_before_libpq_(f):
-        @wraps(f)
-        def skip_before_libpq__(self):
-            v = libpq_version()
-            if v < int("%d%02d%02d" % ver):
-                return self.skipTest("skipped because libpq %d" % v)
-            else:
-                return f(self)
-
-        return skip_before_libpq__
+    def skip_before_libpq_(cls):
+        v = libpq_version()
+        decorator = unittest.skipIf(
+            v < int("%d%02d%02d" % ver),
+            "skipped because libpq %d" % v,
+        )
+        return decorator(cls)
     return skip_before_libpq_
 
 
@@ -338,129 +332,78 @@ def skip_after_libpq(*ver):
     """Skip a test if libpq we're linked to is newer than a certain version."""
     ver = ver + (0,) * (3 - len(ver))
 
-    def skip_after_libpq_(f):
-        @wraps(f)
-        def skip_after_libpq__(self):
-            v = libpq_version()
-            if v >= int("%d%02d%02d" % ver):
-                return self.skipTest("skipped because libpq %s" % v)
-            else:
-                return f(self)
-
-        return skip_after_libpq__
+    def skip_after_libpq_(cls):
+        v = libpq_version()
+        decorator = unittest.skipIf(
+            v >= int("%d%02d%02d" % ver),
+            "skipped because libpq %s" % v,
+        )
+        return decorator(cls)
     return skip_after_libpq_
 
 
 def skip_before_python(*ver):
     """Skip a test on Python before a certain version."""
-    def skip_before_python_(f):
-        @wraps(f)
-        def skip_before_python__(self):
-            if sys.version_info[:len(ver)] < ver:
-                return self.skipTest("skipped because Python %s"
-                    % ".".join(map(str, sys.version_info[:len(ver)])))
-            else:
-                return f(self)
-
-        return skip_before_python__
+    def skip_before_python_(cls):
+        decorator = unittest.skipIf(
+            sys.version_info[:len(ver)] < ver,
+            "skipped because Python %s"
+            % ".".join(map(str, sys.version_info[:len(ver)])),
+        )
+        return decorator(cls)
     return skip_before_python_
 
 
 def skip_from_python(*ver):
     """Skip a test on Python after (including) a certain version."""
-    def skip_from_python_(f):
-        @wraps(f)
-        def skip_from_python__(self):
-            if sys.version_info[:len(ver)] >= ver:
-                return self.skipTest("skipped because Python %s"
-                    % ".".join(map(str, sys.version_info[:len(ver)])))
-            else:
-                return f(self)
-
-        return skip_from_python__
+    def skip_from_python_(cls):
+        decorator = unittest.skipIf(
+            sys.version_info[:len(ver)] >= ver,
+            "skipped because Python %s"
+            % ".".join(map(str, sys.version_info[:len(ver)])),
+        )
+        return decorator(cls)
     return skip_from_python_
 
 
+@decorate_all_tests
 def skip_if_no_superuser(f):
     """Skip a test if the database user running the test is not a superuser"""
     @wraps(f)
     def skip_if_no_superuser_(self):
-        from psycopg2 import ProgrammingError
         try:
             return f(self)
-        except ProgrammingError as e:
-            import psycopg2.errorcodes
-            if e.pgcode == psycopg2.errorcodes.INSUFFICIENT_PRIVILEGE:
-                self.skipTest("skipped because not superuser")
-            else:
-                raise
+        except psycopg2.errors.InsufficientPrivilege:
+            self.skipTest("skipped because not superuser")
 
     return skip_if_no_superuser_
 
 
 def skip_if_green(reason):
-    def skip_if_green_(f):
-        @wraps(f)
-        def skip_if_green__(self):
-            from testconfig import green
-            if green:
-                return self.skipTest(reason)
-            else:
-                return f(self)
-
-        return skip_if_green__
+    def skip_if_green_(cls):
+        decorator = unittest.skipIf(green, reason)
+        return decorator(cls)
     return skip_if_green_
 
+
 skip_copy_if_green = skip_if_green("copy in async mode currently not supported")
 
 
-def skip_if_no_getrefcount(f):
-    @wraps(f)
-    def skip_if_no_getrefcount_(self):
-        if not hasattr(sys, 'getrefcount'):
-            return self.skipTest('skipped, no sys.getrefcount()')
-        else:
-            return f(self)
-    return skip_if_no_getrefcount_
+def skip_if_no_getrefcount(cls):
+    decorator = unittest.skipUnless(
+        hasattr(sys, 'getrefcount'),
+        'no sys.getrefcount()',
+    )
+    return decorator(cls)
 
 
-def skip_if_windows(f):
+def skip_if_windows(cls):
     """Skip a test if run on windows"""
-    @wraps(f)
-    def skip_if_windows_(self):
-        if platform.system() == 'Windows':
-            return self.skipTest("Not supported on Windows")
-        else:
-            return f(self)
-    return skip_if_windows_
-
-
-def script_to_py3(script):
-    """Convert a script to Python3 syntax if required."""
-    if sys.version_info[0] < 3:
-        return script
-
-    import tempfile
-    f = tempfile.NamedTemporaryFile(suffix=".py", delete=False)
-    f.write(script.encode())
-    f.flush()
-    filename = f.name
-    f.close()
-
-    # 2to3 is way too chatty
-    import logging
-    logging.basicConfig(filename=os.devnull)
-
-    from lib2to3.main import main
-    if main("lib2to3.fixes", ['--no-diffs', '-w', '-n', filename]):
-        raise Exception('py3 conversion failed')
-
-    f2 = open(filename)
-    try:
-        return f2.read()
-    finally:
-        f2.close()
-        os.remove(filename)
+    decorator = unittest.skipIf(
+        platform.system() == 'Windows',
+        "Not supported on Windows",
+    )
+    return decorator(cls)
 
 
 class py3_raises_typeerror(object):
@@ -468,7 +411,7 @@ class py3_raises_typeerror(object):
         pass
 
     def __exit__(self, type, exc, tb):
-        if sys.version_info[0] >= 3:
+        if PY3:
             assert type is TypeError
             return True
 
